import Vue from 'vue'
import App from './App.vue'
import router from './router'
import Axios from 'axios'
import SystemInfoService from './api-services/system_info.service'
import DataTypeService from './api-services.gen/wapi.data_type'
import BootstrapVue from 'bootstrap-vue'
import titleMixin from './mixins/title_mixin'
import isMobileMixin from './mixins/is_mobile_mixin'
import {NetvsVuex, watchedMutations} from './vuex'
import NETVSConfig from '@/../netvs.config'

import './prototypes'
import './icons/icons'
import i18n from './i18n'

import vuexUndoRedo from './mixins/vuex-undo-redo'

import APIUtil from './util/apiutil'
import LoginService from '@/api-services/login.service'
import {EventBus} from '@/eventbus'

Vue.mixin(titleMixin)
Vue.mixin(isMobileMixin)
Vue.mixin(vuexUndoRedo)
Vue.use(BootstrapVue)

async function init() {
  const store = NetvsVuex
  let init_fail = false
  let init_fail_info = null
  let session_info = null
  let alerts = []
  const stored_token = JSON.parse(window.localStorage.getItem('token')) || null
  const netdb_axios_config_tmp = {
    headers: {
      Authorization: 'Bearer ' + (stored_token ? stored_token.token : 'INVALID'),
      'Content-Type': 'application/json'
    }
  }
  try {
    alerts = (await SystemInfoService.getAlerts()).data
  } catch {
    // Silent.
  }
  try {
    session_info = (await SystemInfoService.getSessionInfo(netdb_axios_config_tmp)).data
  } catch (e) {
    window.console.error('failed receiving session info')
    if (!(e.response.data instanceof Object) || !('exception' in e.response.data && 'error' in e.response.data.exception) || e.response.data.exception.error_type.name !== 'authentication_error') {
      init_fail = true
    } else {
      window.localStorage.removeItem('token')
    }
    window.console.log(e.response)
    if (e.response.status === 404) {
      init_fail_info = {
        error_type: {
          description: 'API cannot be reached. (404)',
        },
        error: {
          description: 'Make sure your configuration matches the current API version.',
          details: `Current base URL: ${NETVSConfig.NETDB_API_BASE_URL}`
        }
      }
      init_fail = true
    }
  }
  let data_types
  try {
    data_types = APIUtil.dict_by_value_of_array((await DataTypeService.list({}, {})).data[0], 'name')
  } catch (e) {
    data_types = null
    init_fail = true
    if ((typeof e.response.data === 'object' || e.response.data instanceof Object) && 'exception' in e.response.data) {
      init_fail_info = e.response.data.exception
    }
  }

  Vue.config.errorHandler = function (err, vm, info) {
    // handle error
    // `info` is a Vue-specific error info, e.g. which lifecycle hook
    // the error was found in. Only available in 2.2.0+
    // alert(err, info)
    window.console.debug('vue-error handler triggered')
    window.console.log(err, info)
    EventBus.$emit('error', err)
  }
  window.onerror = function (message, source, lineno, colno, error) {
    window.console.debug('window-error handler triggered')
    window.console.log(error)
    EventBus.$emit('error', error)
  }
  window.addEventListener('unhandledrejection', function (event) {
    window.console.debug('unhanldedrejection handler triggered')
    EventBus.$emit('error', event.reason)
  })
  Axios.interceptors.response.use(
    function (response) {
      return response
    },
    async function (error) {
      // handle error
      if (error.response) {
        var stat = error.response.status
        try {
          switch (stat) {
            case 401:
              await LoginService.logout(null)
              store.commit('logout')
              EventBus.$emit('logout')
              if (router.currentRoute.path !== '/login') {
                await router.push('/login')
              }
              break
            case 403:
              await LoginService.logout(null)
              store.commit('logout')
              EventBus.$emit('logout')
              if (router.currentRoute.path !== '/login') {
                await router.push('/login')
              }
              break
            case 400:
              if ('exception' in error.response.data) {
                if (error.response.data.exception.error_type.name === 'authentication_error') {
                  window.console.debug('auth_rej')
                  await LoginService.logout(null)
                  EventBus.$emit('logout')
                  store.commit('logout')
                  if (router.currentRoute.path !== '/login') {
                    await router.push('/login')
                  }
                }
              }
              break
            default:
              break
          }
        } catch (f) {
          window.console.info('silently failed navigation error (probably by calling it in parallel)')
          window.console.info(f)
        }
      } else {
        switch (stat) {
          case 401:
            await LoginService.logout(null)
            EventBus.$emit('logout')
            store.commit('logout')
            router.push('/login')
            break
          default:
            // alert(error)
            window.console.debug(error)
            break
        }
      }
      return Promise.reject(error)
    })
  Vue.config.productionTip = false
  Axios.defaults.baseURL = '/'

  SystemInfoService.getAll().then((sysinfo) => {
    const sysinfo_dat = sysinfo.data
    const mods_by_name = {}
    for (var i = 0; i < sysinfo_dat.mods.length; i++) {
      mods_by_name[sysinfo_dat.mods[i].name] = sysinfo_dat.mods[i]
    }
    window.console.debug('Populating initial vuex state...')
    // Load the current loaded value of i18n
    i18n.locale = store.state.locale
    store.commit('populateInitState', {
      alerts: alerts,
      init_fail: init_fail,
      init_fail_info: init_fail_info,
      xup_perm: session_info ? APIUtil.extract_permissions(session_info.role_list) : {},
      user: session_info ? session_info.mgr_list[0] : null,
      data_types: data_types,
      netdb_axios_config: netdb_axios_config_tmp,
      sysinfo: sysinfo_dat,
      sysinfo_mods_by_name: mods_by_name
    })
    const v = new Vue({
      i18n,
      router,
      store,
      render: h => h(App),
      beforeCreate: function () {
      }
    })
    v.$mount('#app')
    if (v.$store) {
      v.$store.subscribe((mutation, state) => {
        if (watchedMutations.indexOf(mutation.type) >= 0) {
          v.$store.commit('pushUndoRedoDone', mutation)
          if (state.undo_redo_new_mutaion) {
            window.console.debug('New mutation')
            v.$store.commit('clearUndoRedoUndone')
          }
        }
      })
    }
  })
}

init()
